@@ -33,9 +33,22 @@ from autopilot.input import (
     Keyboard,
     Pointer
 )
-from ubiquity_autopilot_tests.emulators import gtktoplevel
-from ubiquity_autopilot_tests.emulators.gtktoplevel import GtkWindow
 from ubiquity_autopilot_tests.emulators import AutopilotGtkEmulatorBase
+from ubiquity_autopilot_tests.emulators.gtkcontainers import (
+    GtkAlignment,
+    GtkBox,
+)
+from ubiquity_autopilot_tests.emulators.gtkcontrols import (
+    GtkButton,
+    GtkCheckButton,
+    GtkLabel,
+    GtkProgressBar,
+    GtkTreeView,
+    GtkRadioButton,
+)
+from ubiquity_autopilot_tests.emulators.gtktoplevel import (
+    GtkWindow,
+)
 from ubiquity_autopilot_tests.testcase import UbiquityTestCase
 from ubiquity_autopilot_tests.configs import english_label_conf
 from ubiquity_autopilot_tests.configs.partconfig import (
@@ -94,7 +107,7 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
 
     @property
     def main_window(self, ):
-        return self.app.select_single('GtkWindow', name='live_installer')
+        return self.app.select_single(GtkWindow, name='live_installer')
 
     def go_to_next_page(self, wait=False):
         """ Goes to the next page of Ubiquity installer
@@ -109,10 +122,29 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
 
         """
         logger.debug('go_to_next_page(wait={0})'.format(wait))
-        nxt_button = self.main_window.select_single('GtkButton', name='next')
+        nxt_button = self.main_window.select_single(GtkButton, name='next')
         nxt_button.click()
 
         if wait:
+            # When the test clicks "Install now" a confirm overwrite disk
+            # dialog appears. This is an "ubi_question_dialog" and although
+            # we can grab it with autopilot we cannot easily select the
+            # continue button as it has no name or BuilderName properties
+            # There are two buttons that basically have identical xpaths
+            # so even identifying the correct button is not really possible.
+            # We experience the same during the manual partition step when
+            # creating a new table. The dialog seems to be a generic msg dialog
+            # that's created on the fly so we can't even add those properties.
+            # The best we can do here is a bit of damage limitation and use a
+            # lengthy sleep and keyboard right to the continue button.
+            print("go_to_next_page() Entering workaround "
+                  "for ubi_question_dialog")
+            print("If the test failed here it is most likely a timing issue")
+            # This maybe overkill but better to be safe
+            time.sleep(40)
+            self.kbd.press_and_release('Right')
+            self.kbd.press_and_release('Enter')
+            print("Workaround ended continuing with wait for next step")
             # This sleep just bridges a weird error when the next button,
             # sometimes flickers its sensitive property back to 1 once clicked
             # and then goes back to 0
@@ -125,10 +157,10 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
                             Eventually(Equals(True), timeout=1200))
 
         page_title = self.main_window.select_single(
-            'GtkLabel', name='page_title')
+            GtkLabel, name='page_title')
         self.assertThat(page_title.label,
                         Eventually(NotEquals(self.current_page_title),
-                                   timeout=120))
+                                   timeout=240))
 
     def go_to_progress_page(self, ):
         """ This simply clicks next and goes to the progress page
@@ -138,7 +170,7 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
             checks.
 
         """
-        nxt_button = self.main_window.select_single('GtkButton', name='next')
+        nxt_button = self.main_window.select_single(GtkButton, name='next')
         nxt_button.click()
 
     def welcome_page_tests(self, lang=None):
@@ -159,8 +191,8 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         # selecting an install language
         logger.debug("Selecting stepLanguage page object")
         welcome_page = self.main_window.select_single(
-            'GtkBox', name='stepLanguage')
-        treeview = welcome_page.select_single('GtkTreeView')
+            GtkBox, name='stepLanguage')
+        treeview = welcome_page.select_single(GtkTreeView)
         # lets get all items
         treeview_items = treeview.get_all_items()
         # first lets check all the items are non-empty unicode strings
@@ -199,6 +231,7 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         self._check_navigation_buttons()
 
     def preparing_page_tests(self, updates=False, thirdParty=False,
+                             minimal_installation=False,
                              networkConnection=True, sufficientSpace=True,
                              powerSource=False):
         """ Runs the tests for the 'Preparing to install' page
@@ -207,6 +240,8 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
 
         :param thirdParty: Boolean, if True selects install third-party
                                     software
+        :param minimal_installation: Boolean, if True selects minimal
+                                       installation option
 
         :param networkConnection: Boolean if True checks the network state box
                                     is visible and objects are correct, If
@@ -231,32 +266,33 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         logger.debug("run_preparing_page_tests()")
         logger.debug("selecting stepPrepare page")
         preparing_page = self.main_window.select_single(
-            'GtkAlignment', BuilderName='stepPrepare')
+            GtkAlignment, BuilderName='stepPrepare')
 
-        objList = ['prepare_best_results', 'prepare_foss_disclaimer',
-                   'prepare_download_updates', 'prepare_nonfree_software']
+        objList = [
+            'prepare_foss_disclaimer',
+            'prepare_minimal_install',
+            'prepare_download_updates',
+            'prepare_nonfree_software'
+        ]
         self.check_visible_object_with_label(objList)
 
         if updates:
             logger.debug("Selecting install updates")
             update_checkbutton = preparing_page.select_single(
-                'GtkCheckButton', BuilderName='prepare_download_updates')
+                GtkCheckButton, BuilderName='prepare_download_updates')
             self.pointing_device.click_object(update_checkbutton)
 
         if thirdParty:
             logger.debug("Selecting install thirdparty software")
             thrdprty_checkbutton = preparing_page.select_single(
-                'GtkCheckButton', BuilderName='prepare_nonfree_software')
+                GtkCheckButton, BuilderName='prepare_nonfree_software')
             self.pointing_device.click_object(thrdprty_checkbutton)
 
-        self._check_preparing_statebox('prepare_network_connection',
-                                       visible=networkConnection)
-        # and sufficient space
-        self._check_preparing_statebox('prepare_sufficient_space',
-                                       visible=sufficientSpace)
-        # and power source
-        self._check_preparing_statebox('prepare_power_source',
-                                       visible=powerSource)
+        if minimal_installation:
+            logger.debug("Selecting install Minimal installation")
+            minimal_installation_checkbutton = preparing_page.select_single(
+                GtkRadioButton, BuilderName='prepare_minimal_install')
+            self.pointing_device.click_object(minimal_installation_checkbutton)
 
         self._check_page_titles()
         self._check_navigation_buttons()
@@ -349,7 +385,7 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         self.check_visible_object_with_label(config.visible_options)
         self.check_hidden_object_with_label(config.hidden_options)
         install_type_page = self.main_window.select_single(
-            'GtkAlignment', BuilderName='stepPartAsk')
+            GtkAlignment, BuilderName='stepPartAsk')
         if option_name:
             obj = install_type_page.select_single(BuilderName=option_name)
             self.pointing_device.click_object(obj)
@@ -372,7 +408,7 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
                      .format(crypto_password))
         logger.debug('Selecting stepPartCrypto page object')
         crypto_page = self.main_window.select_single(
-            'GtkAlignment', BuilderName='stepPartCrypto')
+            GtkAlignment, BuilderName='stepPartCrypto')
 
         items = ['verified_crypto_label', 'crypto_label', 'crypto_description',
                  'crypto_warning', 'crypto_extra_label', 'crypto_extra_time',
@@ -401,8 +437,8 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         logger.debug("run_custom_partition_page_tests()")
         logger.debug("Selecting the stepPartAdvanced page object")
         custom_page = self.main_window.select_single(
-            'GtkAlignment', BuilderName='stepPartAdvanced')
-        treeview = custom_page.select_single('GtkTreeView')
+            GtkAlignment, BuilderName='stepPartAdvanced')
+        treeview = custom_page.select_single(GtkTreeView)
         self.expectThat(treeview.visible, Equals(True),
                         "[Page:'{0}'] Partition tree view was not visible")
         obj_list = ['partition_button_new', 'partition_button_delete',
@@ -450,7 +486,7 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
             partition_dialog.set_file_system_type(elem['FileSystemType'])
             partition_dialog.set_mount_point(elem['MountPoint'])
             ok_button = partition_dialog.select_single(
-                'GtkButton', BuilderName='partition_dialog_okbutton')
+                GtkButton, BuilderName='partition_dialog_okbutton')
             self.pointing_device.click_object(ok_button)
             self.assertThat(partition_dialog.visible,
                             Eventually(Equals(False)),
@@ -475,7 +511,7 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
 
         logger.debug("Selecting stepLocation page object")
         location_page = self.main_window.select_single(
-            'GtkBox', BuilderName='stepLocation')
+            GtkBox, BuilderName='stepLocation')
         location_map = location_page.select_single('CcTimezoneMap')
         self.assertThat(location_map.visible, Equals(True),
                         "Expected location map to be visible but it wasn't")
@@ -496,15 +532,15 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
     def keyboard_layout_page_tests(self, ):
         self._update_current_step('stepKeyboardConf')
         self._check_navigation_buttons(continue_button=True, back_button=True,
-                                       quit_button=False, skip_button=False)
+                                       quit_button=True, skip_button=False)
         self._update_page_titles()
         logger.debug("run_keyboard_layout_page_tests()")
 
         logger.debug("Selecting the stepKeyboardCOnf page object")
         keyboard_page = self.main_window.select_single(
-            'GtkAlignment',
+            GtkAlignment,
             BuilderName='stepKeyboardConf')
-        treeviews = keyboard_page.select_many('GtkTreeView')
+        treeviews = keyboard_page.select_many(GtkTreeView)
         # lets check all the keyboard tree items for the selected language
         # TODO: we should probably test at some point try changing the keyboard
         #       layout to a different language/locale/layout and see if
@@ -549,18 +585,16 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         # TODO: Test detecting keyboard layout
         self._check_page_titles()
         self._check_navigation_buttons(continue_button=True, back_button=True,
-                                       quit_button=False, skip_button=False)
+                                       quit_button=True, skip_button=False)
 
     def user_info_page_tests(self, username, pwd,
-                             encrypted=False, autologin=False):
+                             autologin=False):
         """ Runs tests for the User Info Page
 
         :param username:*String*, name of user
 
         :param pwd: *String*, password for user
 
-        :param encrypted: *Bool* if true encypts the home directory
-
         :param autologin: *Bool* if true sets the user account to login
                            automagically
 
@@ -571,7 +605,7 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         self._update_page_titles()
         logger.debug("Selecting stepUserInfo page")
         user_info_page = self.main_window.select_single(
-            'GtkBox',
+            GtkBox,
             BuilderName='stepUserInfo')
 
         objects = ['hostname_label', 'username_label', 'password_label',
@@ -582,8 +616,6 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
 
         user_info_page.create_user(username, pwd)
         # TODO: get these working
-        if encrypted:
-            user_info_page.encrypt_home_dir(encrypt=True)
         if autologin:
             user_info_page.set_auto_login()
 
@@ -615,7 +647,7 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         # self.expectThat(webkitwindow.visible, Equals(True))
         # print("Webkit window found and is visible")
         print("Selecting Progress bar")
-        progress_bar = self.main_window.select_single('GtkProgressBar',
+        progress_bar = self.main_window.select_single(GtkProgressBar,
                                                       name='install_progress')
 
         # Copying files progress bar
@@ -624,8 +656,9 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
               "progressbar")
         self._track_install_progress()
         print("First loop complete waiting for pbar to go back to 0")
+        # CI VM infrastructure can be a bit slow, hence long timeout.
         self.assertThat(progress_bar.fraction, Eventually(
-            Equals(0.0), timeout=180))
+            Equals(0.0), timeout=530))
         print("Now entering the second loop...........")
         # And now the install progress bar
         self._track_install_progress()
@@ -709,7 +742,7 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         logger.debug("_track_install_progress_bar()")
         print("_track_install_progress()")
         print("selecting progress bar")
-        progress_bar = self.main_window.select_single('GtkProgressBar',
+        progress_bar = self.main_window.select_single(GtkProgressBar,
                                                       name='install_progress')
         progress = 0.0
         while progress < 1.0:
@@ -777,7 +810,7 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
             pass
 
     def _get_dialog_message(self, dlg_object):
-        dialog_labels = dlg_object.select_many('GtkLabel')
+        dialog_labels = dlg_object.select_many(GtkLabel)
         message = ''
         for gtklabel in dialog_labels:
             # only add labels longer than 'Continue' so we avoid button labels
@@ -790,9 +823,9 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         """ adds a new partition """
         logger.debug("_add_new_partition()")
         custom_page = self.main_window.select_single(
-            'GtkAlignment',
+            GtkAlignment,
             BuilderName='stepPartAdvanced')
-        tree_view = custom_page.select_single('GtkTreeView')
+        tree_view = custom_page.select_single(GtkTreeView)
         item = tree_view.select_item(u'  free space')
         self.pointing_device.click_object(item)
         self.assertThat(item.selected, Equals(True),
@@ -810,9 +843,9 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         """
         logger.debug("Checking partition was created.....")
         custom_page = self.main_window.select_single(
-            'GtkAlignment',
+            GtkAlignment,
             BuilderName='stepPartAdvanced')
-        tree_view = custom_page.select_single('GtkTreeView')
+        tree_view = custom_page.select_single(GtkTreeView)
         # assert a new row has been added to the partition table
         total_rows = self._update_table_row_count(config)
         logger.debug("TOTAL NUMBER OF ROWS: {0}".format(self.part_table_rows))
@@ -885,16 +918,16 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         logger.debug("check_window_constants({0}, {1}, {2}, {3})".format(
             continue_button, back_button, quit_button, skip_button))
 
-        con_button = self.main_window.select_single('GtkButton', name='next')
+        con_button = self.main_window.select_single(GtkButton, name='next')
         self.assertThat(con_button.visible, Equals(continue_button))
 
-        bk_button = self.main_window.select_single('GtkButton', name='back')
+        bk_button = self.main_window.select_single(GtkButton, name='back')
         self.assertThat(bk_button.visible, Equals(back_button))
 
-        qt_button = self.main_window.select_single('GtkButton', name='quit')
+        qt_button = self.main_window.select_single(GtkButton, name='quit')
         self.assertThat(qt_button.visible, Equals(quit_button))
 
-        skp_button = self.main_window.select_single('GtkButton', name='skip')
+        skp_button = self.main_window.select_single(GtkButton, name='skip')
         self.assertThat(skp_button.visible, Equals(skip_button))
 
     def _update_current_step(self, name):
@@ -908,9 +941,9 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         " Returns number of rows in table"
 
         custom_page = self.main_window.select_single(
-            'GtkAlignment',
+            GtkAlignment,
             BuilderName='stepPartAdvanced')
-        tree_view = custom_page.select_single('GtkTreeView')
+        tree_view = custom_page.select_single(GtkTreeView)
         num = tree_view.get_number_of_rows()
         if num == self.total_number_partitions:
             # TODO: assert 'free space' changes to a partition
@@ -941,12 +974,12 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
     def _update_page_titles(self, ):
         self.previous_page_title = self.current_page_title
         self.current_page_title = self.main_window.select_single(
-            'GtkLabel',
+            GtkLabel,
             BuilderName='page_title').label
 
     def _check_page_titles(self, ):
         current_page_title = self.main_window.select_single(
-            'GtkLabel',
+            GtkLabel,
             BuilderName='page_title')
         if self.current_step in self.english_config:
             if self.english_install and (
@@ -983,45 +1016,6 @@ class UbiquityAutopilotTestCase(UbiquityTestCase):
         #                     "[Page:'{0}'] Expect page title to be visible "
         #                     "but it wasn't".format(self.current_step))
 
-    def _check_preparing_statebox(self, stateboxName, visible=True,
-                                  imagestock='gtk-yes'):
-        """ Checks the preparing page statebox's """
-        logger.debug("Running checks on {0} StateBox".format(stateboxName))
-        preparing_page = self.main_window.select_single(
-            'GtkAlignment', BuilderName='stepPrepare')
-        state_box = preparing_page.select_single(
-            'StateBox', BuilderName=stateboxName)
-        logger.debug('check({0}, {1})'.format(visible, imagestock))
-        logger.debug("Running checks.......")
-        if visible:
-            self.expectThat(state_box.visible, Equals(visible),
-                            "StateBox.check(): Expected {0} statebox to be "
-                            "visible but it wasn't"
-                            .format(state_box.name))
-            label = state_box.select_single('GtkLabel')
-            self.expectThat(label.label, NotEquals(u''),
-                            "[Page:'{0}'] Expected {1} Statebox's label to "
-                            "contain text but it didn't"
-                            .format(self.current_step, stateboxName))
-            self.expectThat(label.visible, Equals(visible),
-                            "[Page:'{0}'] Expected {1} Statebox label's "
-                            "visible property to be {2} "
-                            .format(self.current_step, stateboxName,
-                                    str(visible)))
-            self.expectIsInstance(label.label, str,
-                                  "[Page:'{0}'] Expected {1} Statebox's label "
-                                  "to be unicode but it wasn't"
-                                  .format(self.current_step, stateboxName))
-            image = state_box.select_single('GtkImage')
-            self.expectThat(image.stock, Equals(imagestock))
-            self.expectThat(image.visible, Equals(visible))
-
-        else:
-            self.expectThat(state_box.visible, Equals(False),
-                            "[Page:'{0}'] Expected {1} statebox to not be "
-                            "visible but it was"
-                            .format(self.current_step, stateboxName))
-
     def get_distribution(self, ):
         """Returns the name of the running distribution."""
         logger.debug("Detecting flavor")
