@@ -15,6 +15,7 @@
 #include <ctype.h>
 #include <signal.h>
 #include <stdarg.h>
+#include <blkid.h>
 
 /**********************************************************************
    Logging
@@ -957,6 +958,10 @@ possible_extended_partition(PedDisk *disk, PedPartition *space)
         result = (ped_disk_type_check_feature(disk->type,
                                               PED_DISK_TYPE_EXTENDED)
                   && !has_extended_partition(disk)
+#ifdef __s390__
+                  /* DASD drives can only do 3 partitions */
+                  && strcmp(disk->dev->model, "IBM S390 DASD drive")
+#endif
                   && possible_primary_partition(disk, space));
         activate_exception_handler();
         return result;
@@ -1330,6 +1335,87 @@ command_dump()
         oprintf("OK\n");
 }
 
+/* Check whether we are running on a sunxi-based, freescale-based, or
+   AM33XX (beaglebone black) system. */
+int
+is_system_with_firmware_on_disk()
+{
+        int cpuinfo_handle;
+        int result = 0;
+        char buf[4096];
+        int length;
+
+        if ((cpuinfo_handle = open("/proc/cpuinfo", O_RDONLY)) != -1) {
+                length = read(cpuinfo_handle, buf, sizeof(buf)-1);
+                if (length > 0) {
+                        buf[length]='\0';
+                        if (strstr(buf, "Allwinner") != NULL)
+                                result = 1;
+                        else if (strstr(buf, "Freescale") != NULL)
+                                result = 1;
+                        else if (strstr(buf, "AM33XX") != NULL)
+                                result = 1;
+                }
+                close(cpuinfo_handle);
+        }
+        return result;
+}
+
+/*
+ * wipe all superblocks libblkid knows about from a device
+ * (condensed from wipefs in util-linux)
+ */
+static int
+do_wipe(char *devname)
+{
+        blkid_probe pr = NULL;
+        int fd = -1;
+        int r = -1;
+
+        fd = open(devname, O_RDWR);
+
+        if (fd < 0) {
+                log("open(%s) failed errno %d", devname, errno);
+                goto error;
+        }
+
+        log("do_wipe open(%s), %d", devname, fd);
+
+
+        pr = blkid_new_probe();
+        if (!pr || blkid_probe_set_device(pr, fd, 0, 0) != 0) {
+                log("setting up probe failed");
+                goto error;
+        }
+
+        blkid_probe_enable_superblocks(pr, 1);
+        blkid_probe_set_superblocks_flags(pr, BLKID_SUBLKS_MAGIC |      /* return magic string and offset */
+                                          BLKID_SUBLKS_TYPE |           /* return superblock type */
+                                          BLKID_SUBLKS_BADCSUM);        /* accept bad checksums */
+
+        blkid_probe_enable_partitions(pr, 1);
+        blkid_probe_set_partitions_flags(pr, BLKID_PARTS_MAGIC |
+                                         BLKID_PARTS_FORCE_GPT);
+
+        while (blkid_do_probe(pr) == 0) {
+                char *type = "unknown";
+                if (blkid_probe_lookup_value(pr, "TYPE", &type, NULL) < 0)
+                        blkid_probe_lookup_value(pr, "PTTYPE", &type, NULL);
+                log("wiping superblock of type %s", type);
+                if (blkid_do_wipe(pr, 0) != 0) {
+                        log("wiping failed");
+                }
+        }
+
+        fsync(fd);
+        r = 0;
+      error:
+        if (fd >= 0)
+                close(fd);
+        blkid_free_probe(pr);
+        return r;
+}
+
 void
 command_commit()
 {
@@ -1337,9 +1423,40 @@ command_commit()
         if (dev == NULL)
                 critical_error("The device %s is not opened.", device_name);
         log("command_commit()");
+
+        /* The boot device on sunxi-based systems needs special handling.
+         * By default partman calls ped_disk_clobber when writing the
+         * partition table, but on sunxi-based systems this would overwrite
+         * the firmware area, resulting in an unbootable system (see
+         * bug #751704).
+         */
+        if (is_system_with_firmware_on_disk() && !strncmp(disk->dev->path, "/dev/mmcblk", 11)) {
+                disk->needs_clobber = 0;
+                log("Sunxi/Freescale/AM33XX detected. Disabling ped_disk_clobber " \
+                    "for the boot device %s to protect the firmware " \
+                    "area.", disk->dev->path);
+        }
+
         open_out();
-        if (disk != NULL && named_is_changed(device_name))
-                ped_disk_commit(disk);
+        if (disk != NULL) {
+                /* ped_disk_clobber does not remove all superblock information
+                 * -- in particular it does not wipe mdraid 0.90 metadata,
+                 * which can lead to an unbootable system as in
+                 * https://bugs.launchpad.net/ubuntu/+source/partman-base/+bug/1828558.
+                 * parted upstream didn't think this was a bug so here we give
+                 * it a helping hand by using libblkid to wipe all superblocks. */
+                if (disk->needs_clobber) {
+                        log("command_commit: wiping superblocks from %s", dev->path);
+                        if (do_wipe(dev->path) != 0) {
+                                log("wiping superblocks from %s failed", dev->path);
+                        } else {
+                                log("wiping superblocks from %s succeeded", dev->path);
+                        }
+                }
+                if (named_is_changed(device_name)) {
+                        ped_disk_commit(disk);
+                }
+        }
         unchange_named(device_name);
         oprintf("OK\n");
 }
@@ -1759,7 +1876,7 @@ command_get_file_system()
                 if (fstype == NULL) {
                         oprintf("none\n");
                 } else {
-                        if (0 == strncmp(part->fs_type->name, "linux-swap", 10))
+                        if (0 == strncmp(fstype->name, "linux-swap", 10))
                                 oprintf("linux-swap\n");
                         else
                                 oprintf("%s\n", fstype->name);
