@@ -220,28 +220,40 @@ class Install(install_misc.InstallBase):
             with open(manifest_remove) as manifest_file:
                 for line in manifest_file:
                     if line.strip() != '' and not line.startswith('#'):
-                        difference.add(line.split()[0])
+                        pkg = line.split(':')[0]
+                        difference.add(pkg.split()[0])
             live_packages = set()
             with open(manifest) as manifest_file:
                 for line in manifest_file:
                     if line.strip() != '' and not line.startswith('#'):
-                        live_packages.add(line.split()[0])
+                        pkg = line.split(':')[0]
+                        live_packages.add(pkg.split()[0])
             desktop_packages = live_packages - difference
         elif os.path.exists(manifest_desktop) and os.path.exists(manifest):
             desktop_packages = set()
             with open(manifest_desktop) as manifest_file:
                 for line in manifest_file:
                     if line.strip() != '' and not line.startswith('#'):
-                        desktop_packages.add(line.split()[0])
+                        pkg = line.split(':')[0]
+                        desktop_packages.add(pkg.split()[0])
             live_packages = set()
             with open(manifest) as manifest_file:
                 for line in manifest_file:
                     if line.strip() != '' and not line.startswith('#'):
-                        live_packages.add(line.split()[0])
+                        pkg = line.split(':')[0]
+                        live_packages.add(pkg.split()[0])
             difference = live_packages - desktop_packages
         else:
             difference = set()
 
+        # Add minimal installation package list if selected
+        if self.db.get('ubiquity/minimal_install') == 'true':
+            if os.path.exists(install_misc.minimal_install_rlist_path):
+                pkgs = set()
+                with open(install_misc.minimal_install_rlist_path) as m_file:
+                    pkgs = {line.strip().split(':')[0] for line in m_file}
+                difference |= pkgs
+
         cache = Cache()
 
         use_restricted = True
@@ -268,24 +280,19 @@ class Install(install_misc.InstallBase):
             if subarch == 'efi':
                 keep.add('grub-efi')
                 keep.add('grub-efi-amd64')
-                efi_vars = "/sys/firmware/efi/vars"
-                sb_var = os.path.join(
-                    efi_vars,
-                    "SecureBoot-8be4df61-93ca-11d2-aa0d-00e098032b8c",
-                    "data")
-                if os.path.exists(sb_var):
-                    with open(sb_var, "rb") as sb_var_file:
-                        if sb_var_file.read(1) == b"\x01":
-                            keep.add('grub-efi-amd64-signed')
-                            keep.add('shim-signed')
-                            try:
-                                altmeta = self.db.get(
-                                    'base-installer/kernel/altmeta')
-                                if altmeta:
-                                    altmeta = '-%s' % altmeta
-                            except debconf.DebconfError:
-                                altmeta = ''
-                            keep.add('linux-signed-generic%s' % altmeta)
+                keep.add('grub-efi-amd64-signed')
+                keep.add('shim-signed')
+                keep.add('mokutil')
+                keep.add('fwupdate-signed')
+                install_misc.record_installed(['fwupdate-signed'])
+                try:
+                    altmeta = self.db.get(
+                        'base-installer/kernel/altmeta')
+                    if altmeta:
+                        altmeta = '-%s' % altmeta
+                except debconf.DebconfError:
+                    altmeta = ''
+                keep.add('linux-signed-generic%s' % altmeta)
             else:
                 keep.add('grub')
                 keep.add('grub-pc')
@@ -387,7 +394,7 @@ class Install(install_misc.InstallBase):
         # remaining" indicator at most every two seconds after that.
 
         copy_progress = 0
-        copied_size, counter = 0, 0
+        copied_size = 0
         directory_times = []
         time_start = time.time()
         times = [(time_start, copied_size)]
@@ -484,6 +491,22 @@ class Install(install_misc.InstallBase):
                     except Exception:
                         # We can live with timestamps being wrong.
                         pass
+                if (hasattr(os, "listxattr") and
+                        hasattr(os, "supports_follow_symlinks") and
+                        os.supports_follow_symlinks):
+                    try:
+                        attrnames = os.listxattr(
+                            sourcepath, follow_symlinks=False)
+                        for attrname in attrnames:
+                            attrvalue = os.getxattr(
+                                sourcepath, attrname, follow_symlinks=False)
+                            os.setxattr(
+                                targetpath, attrname, attrvalue,
+                                follow_symlinks=False)
+                    except OSError as e:
+                        if e.errno not in (
+                                errno.EPERM, errno.ENOTSUP, errno.ENODATA):
+                            raise
 
                 if int((copied_size * 90) / total_size) != copy_progress:
                     copy_progress = int((copied_size * 90) / total_size)
@@ -611,7 +634,7 @@ class Install(install_misc.InstallBase):
                         devbase = sysloop
                     dev = '/dev/%s' % devbase
                     break
-            except:
+            except Exception:
                 continue
 
         if dev == '':
@@ -730,6 +753,7 @@ class Install(install_misc.InstallBase):
                     install_misc.record_installed(['ecryptfs-utils'])
                     break
 
+
 if __name__ == '__main__':
     os.environ['DPKG_UNTRANSLATED_MESSAGES'] = '1'
     if not os.path.exists('/var/lib/ubiquity'):
