@@ -21,16 +21,14 @@
 
 from __future__ import print_function
 
-import fcntl
 import gzip
+import io
 import os
 import platform
 import pwd
 import re
 import shutil
-import socket
 import stat
-import struct
 import subprocess
 import sys
 import syslog
@@ -47,15 +45,6 @@ from ubiquity import install_misc, misc, osextras, plugin_manager
 from ubiquity.components import apt_setup, check_kernels, hw_detect
 
 
-INTERFACES_TEXT = """\
-# This file describes the network interfaces available on your system
-# and how to activate them. For more information, see interfaces(5).
-
-# The loopback network interface
-auto lo
-iface lo inet loopback"""
-
-
 HOSTS_TEXT = """\
 
 # The following lines are desirable for IPv6 capable hosts
@@ -66,12 +55,6 @@ ff02::1 ip6-allnodes
 ff02::2 ip6-allrouters"""
 
 
-IFTAB_TEXT = """\
-# This file assigns persistent names to network interfaces.
-# See iftab(5) for syntax.
-"""
-
-
 def cleanup_after(func):
     def wrapper(self):
         try:
@@ -82,7 +65,7 @@ def cleanup_after(func):
                 self.db.progress('STOP')
             except (KeyboardInterrupt, SystemExit):
                 raise
-            except:
+            except Exception:
                 pass
     return wrapper
 
@@ -105,7 +88,10 @@ class Install(install_misc.InstallBase):
     def __init__(self):
         install_misc.InstallBase.__init__(self)
 
-        self.db = debconf.Debconf()
+        self.db = debconf.Debconf(
+            read=io.TextIOWrapper(sys.stdin.buffer, encoding='utf-8'),
+            write=io.TextIOWrapper(sys.stdout.buffer, encoding='utf-8'))
+
         self.kernel_version = platform.release()
 
         # Get langpacks from install
@@ -238,6 +224,7 @@ class Install(install_misc.InstallBase):
 
         self.next_region()
         self.db.progress('INFO', 'ubiquity/install/bootloader')
+        self.copy_mok()
         self.configure_bootloader()
 
         self.next_region(size=4)
@@ -258,7 +245,7 @@ class Install(install_misc.InstallBase):
         self.db.progress('INFO', 'ubiquity/install/apt_clone_restore')
         try:
             self.apt_clone_restore()
-        except:
+        except Exception:
             syslog.syslog(
                 syslog.LOG_WARNING,
                 'Could not restore packages from the previous install:')
@@ -268,7 +255,7 @@ class Install(install_misc.InstallBase):
             self.db.go()
         try:
             self.copy_network_config()
-        except:
+        except Exception:
             syslog.syslog(
                 syslog.LOG_WARNING,
                 'Could not copy the network configuration:')
@@ -278,7 +265,7 @@ class Install(install_misc.InstallBase):
             self.db.go()
         try:
             self.copy_bluetooth_config()
-        except:
+        except Exception:
             syslog.syslog(
                 syslog.LOG_WARNING,
                 'Could not copy the bluetooth configuration:')
@@ -288,14 +275,14 @@ class Install(install_misc.InstallBase):
             self.db.go()
         try:
             self.recache_apparmor()
-        except:
+        except Exception:
             syslog.syslog(
                 syslog.LOG_WARNING, 'Could not create an Apparmor cache:')
             for line in traceback.format_exc().split('\n'):
                 syslog.syslog(syslog.LOG_WARNING, line)
         try:
             self.copy_wallpaper_cache()
-        except:
+        except Exception:
             syslog.syslog(
                 syslog.LOG_WARNING, 'Could not copy wallpaper cache:')
             for line in traceback.format_exc().split('\n'):
@@ -449,12 +436,6 @@ class Install(install_misc.InstallBase):
                         os.symlink(linkto, targetpath)
                     else:
                         shutil.copy2(path, targetpath)
-        else:
-            if not os.path.exists('/etc/network/interfaces'):
-                # Make sure there's at least something here so that ifupdown
-                # doesn't get upset at boot.
-                with open('/etc/network/interfaces', 'w') as interfaces:
-                    print(INTERFACES_TEXT, file=interfaces)
 
         try:
             hostname = self.db.get('netcfg/get_hostname')
@@ -490,50 +471,6 @@ class Install(install_misc.InstallBase):
             if self.target != '/':
                 shutil.copy2(
                     persistent_net, self.target_file(persistent_net[1:]))
-        else:
-            # TODO cjwatson 2006-03-30: from <bits/ioctls.h>; ugh, but no
-            # binding available
-            SIOCGIFHWADDR = 0x8927
-            # <net/if_arp.h>
-            ARPHRD_ETHER = 1
-
-            if_names = {}
-            sock = socket.socket(socket.SOCK_DGRAM)
-            interfaces = install_misc.get_all_interfaces()
-            for i in range(len(interfaces)):
-                if_names[interfaces[i]] = struct.unpack(
-                    'H6s', fcntl.ioctl(
-                        sock.fileno(), SIOCGIFHWADDR,
-                        struct.pack('256s', interfaces[i].encode()))[16:24])
-            sock.close()
-
-            with open(self.target_file('etc/iftab'), 'w') as iftab:
-                print(IFTAB_TEXT, file=iftab)
-
-                for i in range(len(interfaces)):
-                    dup = False
-
-                    if_name = if_names[interfaces[i]]
-                    if if_name is None or if_name[0] != ARPHRD_ETHER:
-                        continue
-
-                    for j in range(len(interfaces)):
-                        if i == j or if_names[interfaces[j]] is None:
-                            continue
-                        if if_name[1] != if_names[interfaces[j]][1]:
-                            continue
-
-                        if if_names[interfaces[j]][0] == ARPHRD_ETHER:
-                            dup = True
-
-                    if dup:
-                        continue
-
-                    line = (interfaces[i] + " mac " +
-                            ':'.join(['%02x' % if_name[1][c]
-                                      for c in range(6)]))
-                    line += " arp %d" % if_name[0]
-                    print(line, file=iftab)
 
     def run_plugin(self, plugin):
         """Run a single install plugin."""
@@ -816,33 +753,12 @@ class Install(install_misc.InstallBase):
         try:
             if remove_kernels:
                 install_misc.record_removed(remove_kernels, recursive=True)
-        except:
+        except Exception:
             self.db.progress('STOP')
             raise
         self.db.progress('SET', 5)
         self.db.progress('STOP')
 
-    def get_resume_partition(self):
-        biggest_size = 0
-        biggest_partition = None
-        try:
-            with open('/proc/swaps') as swaps:
-                for line in swaps:
-                    words = line.split()
-                    if words[1] != 'partition':
-                        continue
-                    if not os.path.exists(words[0]):
-                        continue
-                    if words[0].startswith('/dev/ramzswap'):
-                        continue
-                    size = int(words[2])
-                    if size > biggest_size:
-                        biggest_size = size
-                        biggest_partition = words[0]
-        except Exception:
-            return None
-        return biggest_partition
-
     def configure_hardware(self):
         """Reconfigure several hardware-specific packages.
 
@@ -870,29 +786,6 @@ class Install(install_misc.InstallBase):
             script += '-oem'
         misc.execute(script)
 
-        resume = self.get_resume_partition()
-        if resume is not None:
-            resume_uuid = None
-            try:
-                resume_uuid = subprocess.Popen(
-                    ['block-attr', '--uuid', resume],
-                    stdout=subprocess.PIPE,
-                    universal_newlines=True).communicate()[0].rstrip('\n')
-            except OSError:
-                pass
-            if resume_uuid:
-                resume = "UUID=%s" % resume_uuid
-            if os.path.exists(self.target_file('etc/initramfs-tools/conf.d')):
-                configdir = self.target_file('etc/initramfs-tools/conf.d')
-            elif os.path.exists(self.target_file('etc/mkinitramfs/conf.d')):
-                configdir = self.target_file('etc/mkinitramfs/conf.d')
-            else:
-                configdir = None
-            if configdir is not None:
-                resume_path = os.path.join(configdir, 'resume')
-                with open(resume_path, 'w') as configfile:
-                    print("RESUME=%s" % resume, file=configfile)
-
         osextras.unlink_force(self.target_file('etc/popularity-contest.conf'))
         try:
             participate = self.db.get('popularity-contest/participate')
@@ -1018,7 +911,8 @@ class Install(install_misc.InstallBase):
 
         inst_boot = self.db.get('ubiquity/install_bootloader')
         if inst_boot == 'true' and 'UBIQUITY_NO_BOOTLOADER' not in os.environ:
-            binds = ("/proc", "/sys", "/dev", "/run")
+            binds = ("/proc", "/sys", "/dev", "/run",
+                     "/sys/firmware/efi/efivars")
             for bind in binds:
                 misc.execute('mount', '--bind', bind, self.target + bind)
 
@@ -1075,6 +969,31 @@ class Install(install_misc.InstallBase):
             for bind in binds:
                 misc.execute('umount', '-f', self.target + bind)
 
+    def copy_mok(self):
+        if 'UBIQUITY_OEM_USER_CONFIG' in os.environ:
+            return
+        try:
+            if self.db.get('oem-config/enable') == 'true':
+                return
+        except debconf.DebconfError:
+            pass
+
+        source = "/var/lib/shim-signed/mok/"
+        target = "/target/var/lib/shim-signed/mok/"
+
+        if not os.path.exists(source):
+            return
+
+        os.makedirs(target, exist_ok=True)
+        for mok_file in os.listdir(source):
+            source_file = os.path.join(source, mok_file)
+            target_file = os.path.join(target, mok_file)
+
+            if os.path.exists(target_file):
+                continue
+
+            shutil.copy(source_file, target_file)
+
     def do_remove(self, to_remove, recursive=False):
         self.nested_progress_start()
 
@@ -1233,6 +1152,8 @@ class Install(install_misc.InstallBase):
         if self.db.get('ubiquity/use_nonfree') == 'true':
             self.db.progress('INFO', 'ubiquity/install/nonfree')
             packages = self.db.get('ubiquity/nonfree_package').split()
+            # also install recorded non-free packages
+            packages.extend(install_misc.query_recorded_installed())
             self.do_install(packages)
 
     def install_extras(self):
@@ -1250,6 +1171,7 @@ class Install(install_misc.InstallBase):
         if not found_cdrom:
             os.rename("%s.apt-setup" % sources_list, sources_list)
 
+        # this will not install non-free packages due to above divert
         self.do_install(install_misc.query_recorded_installed())
 
         if found_cdrom:
@@ -1280,8 +1202,25 @@ class Install(install_misc.InstallBase):
                                 '-o', 'oem', '-g', 'oem',
                                 '/%s' % desktop_file,
                                 '/home/oem/Desktop/%s' % desktop_base)
+                            install_misc.chrex(
+                                self.target,
+                                'sudo', '-i', '-u', 'oem',
+                                'dbus-run-session', '--',
+                                'gio', 'set',
+                                '/home/oem/Desktop/%s' % desktop_base,
+                                'metadata::trusted', 'true')
                             break
 
+                    # Disable gnome-initial-setup for the OEM user
+                    install_misc.chrex(
+                        self.target, 'install', '-d',
+                        '-o', 'oem', '-g', 'oem',
+                        '/home/oem/.config')
+                    install_misc.chrex(
+                        self.target,
+                        'sudo', '-i', '-u', 'oem',
+                        'touch', '/home/oem/.config/gnome-initial-setup-done')
+
                 # Carry the locale setting over to the installed system.
                 # This mimics the behavior in 01oem-config-udeb.
                 di_locale = self.db.get('debian-installer/locale')
@@ -1453,6 +1392,14 @@ class Install(install_misc.InstallBase):
         else:
             difference = set()
 
+        # Add minimal installation package list if selected
+        if self.db.get('ubiquity/minimal_install') == 'true':
+            if os.path.exists(install_misc.minimal_install_rlist_path):
+                rm = set()
+                with open(install_misc.minimal_install_rlist_path) as m_file:
+                    rm = {line.strip().split(':')[0] for line in m_file}
+                difference |= rm
+
         # Keep packages we explicitly installed.
         keep = install_misc.query_recorded_installed()
 
