@@ -1,9 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
-
-from itertools import izip, izip_longest
+from itertools import zip_longest
 import os
-from test import test_support
+from test.support import run_unittest
 import unittest
 
 import debconf
@@ -12,8 +11,10 @@ import mock
 
 from ubiquity import misc, plugin_manager
 
+
 ubi_partman = plugin_manager.load_plugin('ubi-partman')
 
+
 def question_has_variables(question, lookup_variables):
     existing_variables = []
     found_question = False
@@ -21,43 +22,50 @@ def question_has_variables(question, lookup_variables):
         templates_dat = '/var/cache/debconf/templates.dat'
     else:
         templates_dat = 'tests/templates.dat'
-    with open(templates_dat) as templates:
+    # Templates files are (at least in theory) mixed-encoding, so we must
+    # treat them as binary data and decode only marked elements.  In this
+    # function, we only care about question and variable names, which should
+    # always be ASCII.
+    with open(templates_dat, 'rb') as templates:
         for line in templates:
-            if found_question and line == '\n':
+            if found_question and line == b'\n':
                 break
 
-            if line.startswith('Name: %s' % question):
+            if line.startswith(('Name: %s' % question).encode()):
                 found_question = True
                 continue
             elif not found_question:
                 continue
 
-            if line.startswith('Description:'):
+            if line.startswith(b'Description:'):
                 last = 13
-            elif line.startswith('Extended_description:'):
+            elif line.startswith(b'Extended_description:'):
                 last = 22
             else:
                 continue
 
             while True:
-                start = line.find('${', last)
+                start = line.find(b'${', last)
                 if start != -1:
-                    end = line.find('}', last)
+                    end = line.find(b'}', last)
                     if end != -1:
-                        existing_variables.append(line[start+2:end])
+                        existing_variables.append(line[start + 2:end].decode())
                         last = end + 1
                     else:
-                        exc = 'Expected to find } on \'%s\'' % line
-                        raise EOFError, exc
+                        exc = ('Expected to find } on \'%s\'' %
+                               line.decode(errors='replace'))
+                        raise EOFError(exc)
                 else:
                     break
     if not found_question:
-        raise AssertionError, 'Never found the question: %s' % question
+        raise AssertionError('Never found the question: %s' % question)
     only_in_lookup = set(lookup_variables) - set(existing_variables)
     only_in_template = set(existing_variables) - set(lookup_variables)
     if only_in_lookup or only_in_template:
-        raise AssertionError, ('%s\nOnly in lookup: %s\nOnly in template: %s' %
-            (question, ', '.join(only_in_lookup), ', '.join(only_in_template)))
+        raise AssertionError(
+            ('%s\nOnly in lookup: %s\nOnly in template: %s' % (
+                question, ', '.join(only_in_lookup),
+                ', '.join(only_in_template))))
 
 
 # These tests skip when their dependencies are not met and tests/run takes
@@ -83,6 +91,7 @@ class PartmanPageDirectoryTests(unittest.TestCase):
 
     def mock_partman_tree(self):
         prefix = 'tests/partman-tree'
+
         def side_effect_factory(real_method):
             def side_effect(path, *args, **kw):
                 if path.startswith('/lib/partman'):
@@ -101,15 +110,16 @@ class PartmanPageDirectoryTests(unittest.TestCase):
             mocked_method.side_effect = side_effect_factory(real_method)
             self.addCleanup(method.stop)
 
-    #def test_filesystem_description(self):
-    #    for fs in self.page.scripts('/lib/partman/valid_filesystems'):
-    #        print self.page.filesystem_description(fs)
+    # def test_filesystem_description(self):
+    #     for fs in self.page.scripts('/lib/partman/valid_filesystems'):
+    #         print self.page.filesystem_description(fs)
+
+    # def test_method_description(self):
+    #     for method in self.page.subdirectories('/lib/partman/choose_method'):
+    #         if method != 'dont_use':
+    #             self.assertNotEqual(method,
+    #                                 self.page.method_description(method))
 
-    #def test_method_description(self):
-    #    for method in self.page.subdirectories('/lib/partman/choose_method'):
-    #        if method != 'dont_use':
-    #            self.assertNotEqual(method,
-    #                                self.page.method_description(method))
 
 # A couple of mock helpers for some of the tests below.
 def _fake_grub_options(*paths):
@@ -118,11 +128,14 @@ def _fake_grub_options(*paths):
     # element zero.
     def grub_options():
         return [(path,) for path in paths]
+
     return grub_options
 
+
 def _fake_grub_default(default):
     def grub_default():
         return default
+
     return grub_default
 
 
@@ -138,11 +151,12 @@ class TestPageBase(unittest.TestCase):
         # Don't cache descriptions.
         self.page.description_cache = {}
 
+
 class TestPage(TestPageBase):
     def test_description(self):
         question = 'partman-auto/init_automatically_partition'
-        description = unicode(self.page.db.metaget(question, 'description'),
-                              'utf-8', 'replace')
+        description = misc.utf8(self.page.db.metaget(question, 'description'),
+                                'replace')
         self.assertEqual(self.page.description(question), description)
         self.assertIn(question, self.page.description_cache)
 
@@ -162,7 +176,7 @@ class TestPage(TestPageBase):
             default_mountpoints = [default[0] for default in
                                    self.page.default_mountpoint_choices(fs)]
             self.assertTrue(len(default_mountpoints) > 0)
-            self.assertItemsEqual(mountpoints, default_mountpoints)
+            self.assertCountEqual(mountpoints, default_mountpoints)
 
     def test_method_description(self):
         # FIXME: move this into the Directory tests, following use_as()
@@ -193,6 +207,7 @@ class TestPage(TestPageBase):
         no_detected = self.page.extended_description(q)
         self.assertEqual(no_detected, head)
 
+
 @unittest.skipUnless(os.environ['DEB_HOST_ARCH'] in ('amd64', 'i386'),
                      'GRUB-related tests are only relevant on x86')
 class TestPageGrub(TestPageBase):
@@ -218,22 +233,22 @@ class TestPageGrub(TestPageBase):
         self.page.disk_cache = {
             'ignore-1': {
                 'device': '/dev/vda',
-                },
-            }
+            },
+        }
         self.page.partition_cache = {
             'ignore': {
                 'parted': {
                     'path': '/dev/vda1',
                     'fs': 'ext4',
-                    }
                 }
             }
+        }
         self.page.maybe_update_grub()
         self.assertEqual(self.page.ui.set_grub_options.call_count, 1)
         self.page.ui.set_grub_options.assert_called_once_with('/dev/vda', {
             '/dev/vda': True,
             '/dev/vda1': True,
-            })
+        })
 
     @mock.patch('ubiquity.misc.grub_options', _fake_grub_options('/dev/vda1'))
     @mock.patch('ubiquity.misc.grub_default', _fake_grub_default('/dev/vda'))
@@ -243,22 +258,22 @@ class TestPageGrub(TestPageBase):
         self.page.disk_cache = {
             'ignore-1': {
                 'device': '/dev/vda',
-                },
-            }
+            },
+        }
         self.page.partition_cache = {
             'ignore': {
                 'parted': {
                     'path': '/dev/vda1',
                     'fs': 'xfs',
-                    }
                 }
             }
+        }
         self.page.maybe_update_grub()
         self.assertEqual(self.page.ui.set_grub_options.call_count, 1)
         self.page.ui.set_grub_options.assert_called_once_with('/dev/vda', {
             '/dev/vda': True,
             '/dev/vda1': False,
-            })
+        })
 
     @mock.patch('ubiquity.misc.grub_options',
                 _fake_grub_options('/dev/vda1', '/dev/vda2'))
@@ -269,29 +284,29 @@ class TestPageGrub(TestPageBase):
         self.page.disk_cache = {
             'ignore-1': {
                 'device': '/dev/vda',
-                },
-            }
+            },
+        }
         self.page.partition_cache = {
             'ignore-1': {
                 'parted': {
                     'path': '/dev/vda1',
                     'fs': 'xfs',
-                    }
-                },
+                }
+            },
             'ignore-2': {
                 'parted': {
                     'path': '/dev/vda2',
                     'fs': 'ext2',
-                    }
                 }
             }
+        }
         self.page.maybe_update_grub()
         self.assertEqual(self.page.ui.set_grub_options.call_count, 1)
         self.page.ui.set_grub_options.assert_called_once_with('/dev/vda', {
             '/dev/vda': True,
             '/dev/vda1': False,
             '/dev/vda2': True,
-            })
+        })
 
     @mock.patch('ubiquity.misc.grub_options',
                 _fake_grub_options('/dev/vda1', '/dev/vda2', '/dev/vdb1'))
@@ -302,31 +317,31 @@ class TestPageGrub(TestPageBase):
         self.page.disk_cache = {
             'ignore-1': {
                 'device': '/dev/vda',
-                },
+            },
             'ignore-2': {
                 'device': '/dev/vdb',
-                },
-            }
+            },
+        }
         self.page.partition_cache = {
             'ignore-1': {
                 'parted': {
                     'path': '/dev/vda1',
                     'fs': 'xfs',
-                    },
                 },
+            },
             'ignore-2': {
                 'parted': {
                     'path': '/dev/vda2',
                     'fs': 'ext2',
-                    },
                 },
+            },
             'ignore-3': {
                 'parted': {
                     'path': '/dev/vdb1',
                     'fs': 'xfs',
-                    },
                 },
-            }
+            },
+        }
         self.page.maybe_update_grub()
         self.assertEqual(self.page.ui.set_grub_options.call_count, 1)
         self.page.ui.set_grub_options.assert_called_once_with('/dev/vda', {
@@ -335,7 +350,7 @@ class TestPageGrub(TestPageBase):
             '/dev/vda1': False,
             '/dev/vda2': True,
             '/dev/vdb1': False,
-            })
+        })
 
     @mock.patch('ubiquity.misc.grub_options',
                 _fake_grub_options('/dev/vda1', '/dev/vda2', '/dev/vdb1'))
@@ -346,31 +361,31 @@ class TestPageGrub(TestPageBase):
         self.page.disk_cache = {
             'ignore-1': {
                 'device': '/dev/vda',
-                },
+            },
             'ignore-2': {
                 'device': '/dev/vdb',
-                },
-            }
+            },
+        }
         self.page.partition_cache = {
             'ignore-1': {
                 'parted': {
                     'path': '/dev/vda1',
                     'fs': 'ext4',
-                    },
                 },
+            },
             'ignore-2': {
                 'parted': {
                     'path': '/dev/vda2',
                     'fs': 'ext2',
-                    },
                 },
+            },
             'ignore-3': {
                 'parted': {
                     'path': '/dev/vdb1',
                     'fs': 'jfs',
-                    },
                 },
-            }
+            },
+        }
         self.page.maybe_update_grub()
         self.assertEqual(self.page.ui.set_grub_options.call_count, 1)
         self.page.ui.set_grub_options.assert_called_once_with('/dev/vda', {
@@ -379,7 +394,7 @@ class TestPageGrub(TestPageBase):
             '/dev/vda1': True,
             '/dev/vda2': True,
             '/dev/vdb1': False,
-            })
+        })
 
     @mock.patch('ubiquity.misc.grub_options',
                 _fake_grub_options('/dev/vda1', '/dev/vda2', '/dev/vdb1'))
@@ -390,31 +405,31 @@ class TestPageGrub(TestPageBase):
         self.page.disk_cache = {
             'ignore-1': {
                 'device': '/dev/vda',
-                },
+            },
             'ignore-2': {
                 'device': '/dev/vdb',
-                },
-            }
+            },
+        }
         self.page.partition_cache = {
             'ignore-1': {
                 'parted': {
                     'path': '/dev/vda1',
                     'fs': 'ext4',
-                    },
                 },
+            },
             'ignore-2': {
                 'parted': {
                     'path': '/dev/vda2',
                     'fs': 'ext2',
-                    },
                 },
+            },
             'ignore-3': {
                 'parted': {
                     'path': '/dev/vdb1',
                     'fs': 'fat16',
-                    },
                 },
-            }
+            },
+        }
         self.page.maybe_update_grub()
         self.assertEqual(self.page.ui.set_grub_options.call_count, 1)
         self.page.ui.set_grub_options.assert_called_once_with('/dev/vda', {
@@ -423,7 +438,7 @@ class TestPageGrub(TestPageBase):
             '/dev/vda1': True,
             '/dev/vda2': True,
             '/dev/vdb1': True,
-            })
+        })
 
 
 @unittest.skipUnless('DEBCONF_SYSTEMRC' in os.environ, 'Need a database.')
@@ -461,13 +476,15 @@ class TestCalculateAutopartitioningOptions(unittest.TestCase):
 
     # 'This computer currently has Windows on it.'
     def test_windows_only(self):
-        operating_system = u'Windows XP'
+        operating_system = 'Windows XP'
         misc.find_in_os_prober.return_value = operating_system
         part = ubi_partman.Partition('/dev/sda1', 0, '1234-1234', 'ntfs')
-        layout = { '=dev=sda' : [part] }
+        layout = {'=dev=sda': [part]}
         self.page.extra_options = {}
+        self.page.extra_options['use_device'] = ('debconf-return-value',
+                                                 [{'disk-desc': 0}])
         self.page.extra_options['resize'] = {
-            '=dev=sda' : [ '', 0, 0, 0, '', 0, 'ntfs']}
+            '=dev=sda': ['', 0, 0, 0, '', 0, 'ntfs']}
 
         question = 'ubiquity/partitioner/single_os_resize'
         question_has_variables(question, ['OS', 'DISTRO'])
@@ -478,9 +495,8 @@ class TestCalculateAutopartitioningOptions(unittest.TestCase):
         desc = self.page.extended_description(question)
         resize = ubi_partman.PartitioningOption(title, desc)
 
-        question = 'ubiquity/partitioner/single_os_replace'
-        question_has_variables(question, ['OS', 'DISTRO'])
-        self.page.db.subst(question, 'OS', operating_system)
+        question = 'ubiquity/partitioner/multiple_os_format'
+        question_has_variables(question, ['DISTRO'])
         self.page.db.subst(question, 'DISTRO', self.release.name)
         title = self.page.description(question)
         desc = self.page.extended_description(question)
@@ -489,20 +505,20 @@ class TestCalculateAutopartitioningOptions(unittest.TestCase):
         operating_systems, ubuntu_systems = \
             self.page.calculate_operating_systems(layout)
         options = self.page.calculate_autopartitioning_options(
-                        operating_systems, ubuntu_systems)
+            operating_systems, ubuntu_systems)
         self.assertIn('resize', options)
-        self.assertItemsEqual(resize, options['resize'])
+        self.assertCountEqual(resize, options['resize'])
         self.assertIn('use_device', options)
-        self.assertItemsEqual(replace, options['use_device'])
+        self.assertCountEqual(replace, options['use_device'])
         self.assertIn('manual', options)
-        self.assertItemsEqual(self.manual, options['manual'])
+        self.assertCountEqual(self.manual, options['manual'])
 
     # 'This computer currently has no operating systems on it.'
     def test_empty(self):
         self.page.extra_options = {}
         self.page.extra_options['use_device'] = ('debconf-return-value',
                                                  [{'disk-desc': 0}])
-        question = 'ubiquity/partitioner/no_systems_format'
+        question = 'ubiquity/partitioner/multiple_os_format'
         question_has_variables(question, ['DISTRO'])
         self.page.db.subst(question, 'DISTRO', self.release.name)
         title = self.page.description(question)
@@ -511,21 +527,29 @@ class TestCalculateAutopartitioningOptions(unittest.TestCase):
         operating_systems, ubuntu_systems = \
             self.page.calculate_operating_systems([])
         options = self.page.calculate_autopartitioning_options(
-                        operating_systems, ubuntu_systems)
+            operating_systems, ubuntu_systems)
 
         self.assertIn('use_device', options)
-        self.assertItemsEqual(use_device, options['use_device'])
+        self.assertCountEqual(use_device, options['use_device'])
 
         self.assertIn('manual', options)
-        self.assertItemsEqual(self.manual, options['manual'])
+        self.assertCountEqual(self.manual, options['manual'])
 
     # 'This computer currently has Ubuntu 10.04 on it.'
     def test_older_ubuntu_only(self):
-        operating_system = u'Ubuntu 10.04'
-        misc.find_in_os_prober.return_value = operating_system
+        operating_system = 'Ubuntu 10.04'
+        operating_version = '10.04'
+
+        def side_effect(*args, **kwargs):
+            if 'with_version' in kwargs:
+                return operating_system, operating_version
+            return operating_system
+
+        misc.find_in_os_prober.side_effect = side_effect
         part = ubi_partman.Partition('/dev/sda1', 0, '1234-1234', 'ext4')
-        layout = { '=dev=sda' : [part] }
+        layout = {'=dev=sda': [part]}
         self.page.extra_options = {}
+        self.page.extra_options['replace'] = ['/dev/sda1']
         self.page.extra_options['use_device'] = ('debconf-return-value',
                                                  [{'disk-desc': 0}])
         self.page.extra_options['reuse'] = [(0, '/dev/sda1')]
@@ -535,36 +559,44 @@ class TestCalculateAutopartitioningOptions(unittest.TestCase):
         self.page.db.subst(question, 'CURDISTRO', operating_system)
         title = self.page.description(question)
         desc = self.page.extended_description(question)
+        replace = ubi_partman.PartitioningOption(title, desc)
+
+        question = 'ubiquity/partitioner/multiple_os_format'
+        question_has_variables(question, ['DISTRO'])
+        self.page.db.subst(question, 'DISTRO', self.release.name)
+        title = self.page.description(question)
+        desc = self.page.extended_description(question)
         use_device = ubi_partman.PartitioningOption(title, desc)
 
-        question = 'ubiquity/partitioner/ubuntu_upgrade'
-        question_has_variables(question, ['CURDISTRO', 'VER'])
-        self.page.db.subst(question, 'CURDISTRO', operating_system)
-        self.page.db.subst(question, 'VER', self.release.version)
-        title = self.page.description(question)
-        desc = self.page.extended_description(question)
-        reuse = ubi_partman.PartitioningOption(title, desc)
-
         operating_systems, ubuntu_systems = \
             self.page.calculate_operating_systems(layout)
         options = self.page.calculate_autopartitioning_options(
-                        operating_systems, ubuntu_systems)
+            operating_systems, ubuntu_systems)
+
+        self.assertIn('replace', options)
+        self.assertCountEqual(replace, options['replace'])
+
         self.assertIn('use_device', options)
-        self.assertItemsEqual(use_device, options['use_device'])
+        self.assertCountEqual(use_device, options['use_device'])
 
         self.assertIn('manual', options)
-        self.assertItemsEqual(self.manual, options['manual'])
-
-        self.assertIn('reuse', options)
-        self.assertItemsEqual(reuse, options['reuse'])
+        self.assertCountEqual(self.manual, options['manual'])
 
     # 'This computer currently has Ubuntu 12.04 on it.'
     def test_same_ubuntu_only(self):
-        operating_system = u'Ubuntu 12.04'
-        misc.find_in_os_prober.return_value = operating_system
+        operating_system = 'Ubuntu 12.04'
+        operating_version = '12.04'
+
+        def side_effect(*args, **kwargs):
+            if 'with_version' in kwargs:
+                return operating_system, operating_version
+            return operating_system
+
+        misc.find_in_os_prober.side_effect = side_effect
         part = ubi_partman.Partition('/dev/sda1', 0, '1234-1234', 'ext4')
-        layout = { '=dev=sda' : [part] }
+        layout = {'=dev=sda': [part]}
         self.page.extra_options = {}
+        self.page.extra_options['replace'] = ['/dev/sda1']
         self.page.extra_options['use_device'] = ('debconf-return-value',
                                                  [{'disk-desc': 0}])
         self.page.extra_options['reuse'] = [(0, '/dev/sda1')]
@@ -574,6 +606,13 @@ class TestCalculateAutopartitioningOptions(unittest.TestCase):
         self.page.db.subst(question, 'CURDISTRO', operating_system)
         title = self.page.description(question)
         desc = self.page.extended_description(question)
+        replace = ubi_partman.PartitioningOption(title, desc)
+
+        question = 'ubiquity/partitioner/multiple_os_format'
+        question_has_variables(question, ['DISTRO'])
+        self.page.db.subst(question, 'DISTRO', self.release.name)
+        title = self.page.description(question)
+        desc = self.page.extended_description(question)
         use_device = ubi_partman.PartitioningOption(title, desc)
 
         question = 'ubiquity/partitioner/ubuntu_reinstall'
@@ -585,22 +624,34 @@ class TestCalculateAutopartitioningOptions(unittest.TestCase):
         operating_systems, ubuntu_systems = \
             self.page.calculate_operating_systems(layout)
         options = self.page.calculate_autopartitioning_options(
-                        operating_systems, ubuntu_systems)
+            operating_systems, ubuntu_systems)
+
+        self.assertIn('replace', options)
+        self.assertCountEqual(replace, options['replace'])
+
         self.assertIn('use_device', options)
-        self.assertItemsEqual(use_device, options['use_device'])
+        self.assertCountEqual(use_device, options['use_device'])
 
         self.assertIn('manual', options)
-        self.assertItemsEqual(self.manual, options['manual'])
+        self.assertCountEqual(self.manual, options['manual'])
 
         self.assertNotIn('reuse', options)
 
     # 'This computer currently has Ubuntu 90.10 on it.'
     def test_newer_ubuntu_only(self):
-        operating_system = u'Ubuntu 90.10'
-        misc.find_in_os_prober.return_value = operating_system
+        operating_system = 'Ubuntu 90.10'
+        operating_version = '90.10'
+
+        def side_effect(*args, **kwargs):
+            if 'with_version' in kwargs:
+                return operating_system, operating_version
+            return operating_system
+
+        misc.find_in_os_prober.side_effect = side_effect
         part = ubi_partman.Partition('/dev/sda1', 0, '1234-1234', 'ext4')
-        layout = { '=dev=sda' : [part] }
+        layout = {'=dev=sda': [part]}
         self.page.extra_options = {}
+        self.page.extra_options['replace'] = ['/dev/sda1']
         self.page.extra_options['use_device'] = ('debconf-return-value',
                                                  [{'disk-desc': 0}])
         self.page.extra_options['reuse'] = [(0, '/dev/sda1')]
@@ -610,6 +661,13 @@ class TestCalculateAutopartitioningOptions(unittest.TestCase):
         self.page.db.subst(question, 'CURDISTRO', operating_system)
         title = self.page.description(question)
         desc = self.page.extended_description(question)
+        replace = ubi_partman.PartitioningOption(title, desc)
+
+        question = 'ubiquity/partitioner/multiple_os_format'
+        question_has_variables(question, ['DISTRO'])
+        self.page.db.subst(question, 'DISTRO', self.release.name)
+        title = self.page.description(question)
+        desc = self.page.extended_description(question)
         use_device = ubi_partman.PartitioningOption(title, desc)
 
         question = 'ubiquity/partitioner/ubuntu_reinstall'
@@ -621,30 +679,36 @@ class TestCalculateAutopartitioningOptions(unittest.TestCase):
         operating_systems, ubuntu_systems = \
             self.page.calculate_operating_systems(layout)
         options = self.page.calculate_autopartitioning_options(
-                        operating_systems, ubuntu_systems)
+            operating_systems, ubuntu_systems)
+
+        self.assertIn('replace', options)
+        self.assertCountEqual(replace, options['replace'])
+
         self.assertIn('use_device', options)
-        self.assertItemsEqual(use_device, options['use_device'])
+        self.assertCountEqual(use_device, options['use_device'])
 
         self.assertIn('manual', options)
-        self.assertItemsEqual(self.manual, options['manual'])
+        self.assertCountEqual(self.manual, options['manual'])
 
         self.assertNotIn('reuse', options)
 
     # 'This computer currently has multiple operating systems on it.'
     def test_multiple_operating_systems(self):
-        operating_systems = [u'Ubuntu 10.04', u'Windows XP', u'Mac OSX']
+        operating_systems = ['Ubuntu 10.04', 'Windows XP', 'Mac OSX']
+
         def side_effect(*args, **kwargs):
             return operating_systems.pop()
+
         misc.find_in_os_prober.side_effect = side_effect
         part1 = ubi_partman.Partition('/dev/sda1', 0, '1234-1234', 'ext4')
         part2 = ubi_partman.Partition('/dev/sda2', 0, '1234-1234', 'ext4')
         part3 = ubi_partman.Partition('/dev/sda3', 0, '1234-1234', 'ext4')
-        layout = { '=dev=sda' : [part1, part2, part3] }
+        layout = {'=dev=sda': [part1, part2, part3]}
         self.page.extra_options = {}
         self.page.extra_options['use_device'] = ('debconf-return-value',
                                                  [{'disk-desc': 0}])
         self.page.extra_options['resize'] = {
-            '=dev=sda' : [ '', 0, 0, 0, '', 0, 'ntfs']}
+            '=dev=sda': ['', 0, 0, 0, '', 0, 'ntfs']}
 
         question = 'ubiquity/partitioner/multiple_os_format'
         question_has_variables(question, ['DISTRO'])
@@ -663,15 +727,15 @@ class TestCalculateAutopartitioningOptions(unittest.TestCase):
         operating_systems, ubuntu_systems = \
             self.page.calculate_operating_systems(layout)
         options = self.page.calculate_autopartitioning_options(
-                        operating_systems, ubuntu_systems)
+            operating_systems, ubuntu_systems)
         self.assertIn('use_device', options)
-        self.assertItemsEqual(use_device, options['use_device'])
+        self.assertCountEqual(use_device, options['use_device'])
 
         self.assertIn('resize', options)
-        self.assertItemsEqual(resize, options['resize'])
+        self.assertCountEqual(resize, options['resize'])
 
         self.assertIn('manual', options)
-        self.assertItemsEqual(self.manual, options['manual'])
+        self.assertCountEqual(self.manual, options['manual'])
 
 
 def _fake_grub_options_pairs(paths, descriptions):
@@ -681,20 +745,23 @@ def _fake_grub_options_pairs(paths, descriptions):
     def grub_options():
         return [(path, description)
                 for path, description
-                in izip_longest(paths, descriptions, fillvalue='')]
+                in zip_longest(paths, descriptions, fillvalue='')]
     return grub_options
 
+
 class TestPageGtk(unittest.TestCase):
     def setUp(self):
         # Without this, GtkBuilder cannot construct ResizeWidget and
         # PartitionBox widgets.
         from ubiquity import gtkwidgets
-        gtkwidgets # pacify pyflakes
+
+        gtkwidgets  # pacify pyflakes
         controller = mock.Mock()
         self.gtk = ubi_partman.PageGtk(controller)
 
     def test_advanced_page_link(self):
         from ubiquity import gtkwidgets
+
         self.gtk.part_auto_hidden_label.emit('activate-link', '')
         gtkwidgets.refresh()
         self.gtk.controller.go_forward.assert_called_once_with()
@@ -714,26 +781,26 @@ class TestPageGtk(unittest.TestCase):
             '/dev/vda2': False,
             '/dev/vdb': True,
             '/dev/vdb1': True,
-            })
+        })
         # The combo box should have everything but vda2.
         expected = [
             '/dev/vda Virtio Block Device (108 GB)',
             '/dev/vdb Virtio Block Device (801 GB)',
             '/dev/vda1 ',
             '/dev/vdb1 ',
-            ]
+        ]
         row_text = []
         for row in self.gtk.grub_device_entry.get_model():
             row_text.append(' '.join(row))
-        for want, got in izip(expected, row_text):
+        for want, got in zip(expected, row_text):
             self.assertEqual(want, got)
 
 
 if __name__ == '__main__':
-    test_support.run_unittest(
+    run_unittest(
         TestCalculateAutopartitioningOptions,
         TestPage,
         TestPageGrub,
         TestPageGtk,
         PartmanPageDirectoryTests,
-        )
+    )
