@@ -45,16 +45,17 @@ import syslog
 import traceback
 
 import dbus
+assert dbus  # silence, pyflakes!
 from dbus.mainloop.glib import DBusGMainLoop
 DBusGMainLoop(set_as_default=True)
 
 # in query mode we won't be in X, but import needs to pass
 if 'DISPLAY' in os.environ:
-    from gi.repository import Gtk, Gdk, GObject, GLib, Atk
+    from gi.repository import Gtk, Gdk, GObject, GLib, Atk, Gio
     from ubiquity import gtkwidgets
 
 from ubiquity import (
-    filteredcommand, gsettings, i18n, validation, misc, osextras)
+    filteredcommand, gsettings, i18n, validation, misc, osextras, telemetry)
 from ubiquity.components import install, plugininstall, partman_commit
 import ubiquity.frontend.base
 from ubiquity.frontend.base import BaseFrontend
@@ -159,6 +160,13 @@ class Controller(ubiquity.frontend.base.Controller):
         self._wizard.switch_to_install_interface()
 
 
+def on_screen_reader_enabled_changed(gsettings, key):
+    # handle starting orca only, it exits itself when the key is false
+    if (key == "screen-reader-enabled" and gsettings.get_boolean(key) and
+       osextras.find_on_path('orca')):
+        subprocess.Popen(['orca'], preexec_fn=misc.drop_all_privileges)
+
+
 class Wizard(BaseFrontend):
     def __init__(self, distro):
         def add_subpage(self, steps, name):
@@ -234,6 +242,7 @@ class Wizard(BaseFrontend):
         self.timeout_id = None
         self.screen_reader = False
         self.orca_process = None
+        self.a11y_settings = None
 
         # To get a "busy mouse":
         self.watch = Gdk.Cursor.new(Gdk.CursorType.WATCH)
@@ -245,6 +254,22 @@ class Wizard(BaseFrontend):
             if 'access=v3' in fp.read():
                 self.screen_reader = True
 
+        if 'UBIQUITY_ONLY' in os.environ:
+            # do not run this as root. The API pretends to be synchronous but
+            # it is actually asynchronous. If you become root before it
+            # finishes then D-Bus will reject our connection due to a
+            # mismatched user between the requestor and the owner of the
+            # session bus.
+
+            # handle orca only in ubiquity-dm where there is no gnome-session
+            self.a11y_settings = Gio.Settings.new(
+                "org.gnome.desktop.a11y.applications")
+            self.a11y_settings.connect("changed::screen-reader-enabled",
+                                       on_screen_reader_enabled_changed)
+            # enable if needed and a key read is needed to connect the signal
+            on_screen_reader_enabled_changed(self.a11y_settings,
+                                             "screen-reader-enabled")
+
         # set default language
         self.locale = i18n.reset_locale(self)
 
@@ -256,18 +281,24 @@ class Wizard(BaseFrontend):
         # Make a thin Progress bar
         provider = Gtk.CssProvider()
         provider.load_from_data(b'''\
-            .inline-toolbar.toolbar {
+            toolbar {
                 background: @theme_bg_color;
                 border-color: transparent;
                 border-width: 0px;
                 padding: 0px;
             }
-            GtkProgressBar {
-              -GtkProgressBar-min-horizontal-bar-height : 10;
-              -GtkProgressBar-min-horizontal-bar-width : 10;
+            progressbar trough {
+              min-height: 10px;
+              min-width: 11px;
+              border: none;
             }
-            GtkPaned {
-                -GtkPaned-handle-size: 10;
+            progressbar progress {
+              min-height: 10px;
+              border-radius: 4px;
+              border: none;
+            }
+            paned separator {
+                min-width: 10px;
             }
             ''')
         Gtk.StyleContext.add_provider_for_screen(
@@ -352,16 +383,17 @@ class Wizard(BaseFrontend):
             try:
                 subprocess.Popen(['a11y-profile-manager-indicator',
                                   '-i'], preexec_fn=misc.drop_all_privileges)
-                if osextras.find_on_path('canberra-gtk-play'):
-                    subprocess.Popen(
-                        ['canberra-gtk-play', '--id=system-ready'],
-                        preexec_fn=misc.drop_all_privileges)
-            except:
+            except Exception:
                 print("Unable to set up accessibility profile support",
                       file=sys.stderr)
             self.live_installer.connect(
                 'key-press-event', self.a11y_profile_keys)
 
+        if osextras.find_on_path('canberra-gtk-play'):
+            subprocess.Popen(
+                ['canberra-gtk-play', '--id=system-ready'],
+                preexec_fn=misc.drop_all_privileges)
+
     def all_children(self, parent):
         if isinstance(parent, Gtk.Container):
             def recurse(x, y):
@@ -503,7 +535,7 @@ class Wizard(BaseFrontend):
                 os.rename('%s.new' % thunar_volmanrc, thunar_volmanrc)
             except (KeyboardInterrupt, SystemExit):
                 raise
-            except:
+            except Exception:
                 pass
         return previous
 
@@ -530,31 +562,6 @@ class Wizard(BaseFrontend):
 
         gsettings.set(gs_schema, gs_key, gs_value)
 
-    def disable_screen_reader(self):
-        gs_key = 'screenreader'
-        for gs_schema in 'org.gnome.settings-daemon.plugins.media-keys', \
-                         'org.mate.SettingsDaemon.plugins.media-keys':
-            gs_previous = '%s/%s' % (gs_schema, gs_key)
-            if gs_previous in self.gsettings_previous:
-                return
-
-            gs_value = gsettings.get(gs_schema, gs_key)
-            self.gsettings_previous[gs_previous] = gs_value
-
-            if gs_value:
-                gsettings.set(gs_schema, gs_key, '')
-
-        atexit.register(self.enable_screen_reader)
-
-    def enable_screen_reader(self):
-        gs_key = 'screenreader'
-        for gs_schema in 'org.gnome.settings-daemon.plugins.media-keys', \
-                         'org.mate.SettingsDaemon.plugins.media-keys':
-            gs_previous = '%s/%s' % (gs_schema, gs_key)
-            gs_value = self.gsettings_previous[gs_previous]
-
-            gsettings.set(gs_schema, gs_key, gs_value)
-
     def disable_screensaver(self):
         gs_schema = 'org.gnome.desktop.screensaver'
         gs_key = 'idle-activation-enabled'
@@ -730,7 +737,6 @@ class Wizard(BaseFrontend):
         self.disable_volume_manager()
         self.disable_screensaver()
         self.disable_powermgr()
-        self.disable_screen_reader()
 
         if 'UBIQUITY_ONLY' in os.environ:
             self.disable_logout_indicator()
@@ -750,6 +756,9 @@ class Wizard(BaseFrontend):
             self.debconf_progress_cancellable(False)
             self.refresh()
 
+        telemetry.get().set_installer_type('GTK')
+        telemetry.get().set_is_oem(self.oem_config)
+
         self.set_current_page(0)
         self.live_installer.show()
 
@@ -814,6 +823,8 @@ class Wizard(BaseFrontend):
         # postinstall will exit here by calling Gtk.main_quit in
         # find_next_step.
 
+        telemetry.get().done(self.db)
+
         self.unlock_environment()
         if self.oem_user_config:
             self.quit_installer()
@@ -863,6 +874,7 @@ class Wizard(BaseFrontend):
         misc.drop_privileges_save()
         self.progress_mode.set_current_page(
             self.progress_pages['progress_bar'])
+        telemetry.get().add_stage('user_done')
 
         if not self.slideshow:
             self.page_mode.hide()
@@ -1002,7 +1014,7 @@ class Wizard(BaseFrontend):
                 cfg.read(os.path.join(self.slideshow, 'slideshow.conf'))
                 config_width = int(cfg.get('Slideshow', 'width'))
                 config_height = int(cfg.get('Slideshow', 'height'))
-            except:
+            except Exception:
                 config_width = 752
                 config_height = 442
             self.webkit_scrolled_window.set_size_request(
@@ -1404,45 +1416,23 @@ class Wizard(BaseFrontend):
 
     def do_reboot(self):
         """Callback for main program to actually reboot the machine."""
-        try:
-            session = dbus.Bus.get_session()
-            gnome_session = session.name_has_owner('org.gnome.SessionManager')
-        except dbus.exceptions.DBusException:
-            gnome_session = False
-
-        if gnome_session:
-            manager = session.get_object('org.gnome.SessionManager',
-                                         '/org/gnome/SessionManager')
-            manager.RequestReboot()
-        else:
-            # don't let reboot race with the shutdown of X in ubiquity-dm;
-            # reboot might be too fast and X will stay around forever instead
-            # of moving to plymouth
-            misc.execute_root(
-                "sh", "-c",
-                "if ! service display-manager status; then killall Xorg; "
-                "while pidof X; do sleep 0.5; done; fi; reboot")
+        # don't let reboot race with the shutdown of X in ubiquity-dm;
+        # reboot might be too fast and X will stay around forever instead
+        # of moving to plymouth
+        misc.execute_root(
+            "sh", "-c",
+            "if ! service display-manager status; then killall Xorg; "
+            "while pidof X; do sleep 0.5; done; fi; reboot")
 
     def do_shutdown(self):
         """Callback for main program to actually shutdown the machine."""
-        try:
-            session = dbus.Bus.get_session()
-            gnome_session = session.name_has_owner('org.gnome.SessionManager')
-        except dbus.exceptions.DBusException:
-            gnome_session = False
-
-        if gnome_session:
-            manager = session.get_object('org.gnome.SessionManager',
-                                         '/org/gnome/SessionManager')
-            manager.RequestShutdown()
-        else:
-            # don't let poweroff race with the shutdown of X in ubiquity-dm;
-            # poweroff might be too fast and X will stay around forever instead
-            # of moving to plymouth
-            misc.execute_root(
-                "sh", "-c",
-                "if ! service display-manager status; then killall Xorg; "
-                "while pidof X; do sleep 0.5; done; fi; poweroff")
+        # don't let poweroff race with the shutdown of X in ubiquity-dm;
+        # poweroff might be too fast and X will stay around forever instead
+        # of moving to plymouth
+        misc.execute_root(
+            "sh", "-c",
+            "if ! service display-manager status; then killall Xorg; "
+            "while pidof X; do sleep 0.5; done; fi; poweroff")
 
     def quit_installer(self, *args):
         """Quit installer cleanly."""
@@ -1548,6 +1538,7 @@ class Wizard(BaseFrontend):
         for i in range(len(self.pages))[index + 1:]:
             self.dot_grid.get_child_at(i, 0).set_fraction(0)
 
+        telemetry.get().add_stage(name)
         syslog.syslog('switched to page %s' % name)
 
     # Callbacks provided to components.
@@ -1638,6 +1629,8 @@ class Wizard(BaseFrontend):
             return False
 
     def switch_to_install_interface(self):
+        if not self.installing:
+            telemetry.get().add_stage(telemetry.START_INSTALL_STAGE_TAG)
         self.installing = True
         self.lockdown_environment()
 
@@ -1773,9 +1766,9 @@ class Wizard(BaseFrontend):
             self.grub_new_device_entry.set_sensitive(True)
 
     def bootloader_dialog(self, current_device):
-        l = self.skip_label.get_label()
-        l = l.replace('${RELEASE}', misc.get_release().name)
-        self.skip_label.set_label(l)
+        ret = self.skip_label.get_label()
+        ret = ret.replace('${RELEASE}', misc.get_release().name)
+        self.skip_label.set_label(ret)
         self.grub_new_device_entry.get_child().set_text(current_device)
         self.grub_new_device_entry.get_child().grab_focus()
         response = self.bootloader_fail_dialog.run()
